/****************************************************************************\
*   TWIN - this header file is part of Binary vIEW project (BIEW)            *
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
*   Copyrights:           1995, 2000 Nick Kurshev                            *
*   License:              See below                                          *
*   Author and developer: Nick Kurshev                                       *
*   Requirement:          ANSI C compatible development system               *
*   Original file name:   lib/twin.h                                         *
*---------------------- [ D e s c r i p t i o n ] ---------------------------*
*  This file contain prototypes of Text Window manager functions.            *
*-------------------------- [ L i c e n c e ] -------------------------------*
* The Binary vIEW (BIEW) is copyright (C) 1995 Nick Kurshev.                 *
* All rights reserved. This software is redistributable under the licence    *
* given in the file "Licence.en" ("Licence.ru" in russian translation)       *
* distributed in the BIEW archive.                                           *
\****************************************************************************/

#ifndef __TWIN_H
#define __TWIN_H 1

#ifndef __BIEWLIB_H
#include "lib/biewlib.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef enum
{
  Black = 0,
  Blue,
  Green,
  Cyan,
  Red,
  Magenta,
  Brown,
  LightGray,
  Gray,
  LightBlue,
  LightGreen,
  LightCyan,
  LightRed,
  LightMagenta,
  Yellow,
  White
}Color;

/* Types */

typedef unsigned char tRelCoord;
typedef unsigned char tAbsCoord;
typedef enum
{
  TW_TMODE_LEFT = 0,
  TW_TMODE_CENTER,
  TW_TMODE_RIGHT
}tTitleMode;

/* Flags of window */

#define TWS_NONE           0x0000
#define TWS_FRAMEABLE      0x0001
#define TWS_VISIBLE        0x0002
#define TWS_CURSORABLE     0x0004
#define TWS_NLS866         0x0100

#define TW_SINGLE_FRAME "Ŀ"
#define TW_DOUBLE_FRAME "ͻͼ"

#define TW_THIN_FRAME   TW_SINGLE_FRAME
#define TW_MEDIUM_FRAME ""
#define TW_THICK_FRAME  ""

#define TW_UP3D_FRAME   "\x00\x00\x00\x00\x00\x00\x00\x00"
#define TW_DN3D_FRAME   "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF"

typedef unsigned char ColorAttr;

typedef struct tagDefColor
{
  ColorAttr user;
  ColorAttr system;
}DefColor;

#define LOGFB_TO_PHYS(fore,back) ((back << 4) | fore)
#define PHYS_TO_LOGFB(attr,fore,back) { register ColorAttr a; a = attr; fore = a & 0x0F; back = (a >> 4) & 0x0F; }
#define BACK_COLOR(attr) ((attr >> 4) & 0x0F)
#define FORE_COLOR(attr) (attr & 0x0F)

/*
   This struct is ordered as it documented in Athlon manual
   Publication # 22007 Rev: D
*/
typedef struct tagTWindow
{
  char           Frame[8];    /* Buffer, contaning frame component */
                              /* Frame encoding:   1---2---3 */
                              /*                   |       | */
                              /*                   4       5 */
                              /*                   |       | */
                              /*                   6---7---8 */
  unsigned long  iflags;      /* contain internal flags of window state */
  struct tagTWindow *next;
  void *         usrData;     /* user data pointer can contain in windows */
  char *         body;        /* Buffer, containing the image of window frame */
  char *         saved;       /* Buffer, containing the saved image under window */
  char *         Title;       /* Caption of window */
  char *         Footer;      /* Footer of window */
  unsigned       wsize;       /* Size of buffers in bytes */
  unsigned       wwidth;      /* width of window */
  unsigned       wheight;     /* height of window */
  unsigned       flags;       /* Window flags */
  tRelCoord      X1,Y1,X2,Y2; /* coordinates of window on the screen */
  tRelCoord      cur_x,cur_y; /* coordinates of cursor position inside the window */
  tTitleMode     TitleMode;   /* Mode displaying of title */
  tTitleMode     FooterMode;  /* Mode displaying of footer */
  DefColor       text;        /* default color of text */
  DefColor       frame;       /* default color of frame */
  DefColor       title;       /* default color of title */
  DefColor       footer;      /* default color of footer text */
}TWindow;

void __FASTCALL__ twinDrawFrame(tRelCoord x1, tRelCoord y1, tRelCoord x2, tRelCoord y2,const char *frame,Color fore, Color back);
void __FASTCALL__ twinDrawFrameAttr(tRelCoord x1, tRelCoord y1, tRelCoord x2, tRelCoord y2,const char *frame,ColorAttr attr);


/* Call this function before any other */

#define TWIF_FORCEMONO   0x00000001L

void              __FASTCALL__ twInit( unsigned long vio_flags, unsigned long twin_flags );
/* Call this function after all other */
void              __FASTCALL__ twDestroy( void );

TWindow *         __FASTCALL__ twCreateWin(tAbsCoord x1, tAbsCoord y1, tAbsCoord width, tAbsCoord height, unsigned flags);
void              __FASTCALL__ twDestroyWin(TWindow *);

void              __FASTCALL__ twHideWin(TWindow *);
void              __FASTCALL__ twShowWin(TWindow *);
void              __FASTCALL__ twShowWinBeneath(TWindow *win,TWindow *prev);
void              __FASTCALL__ twShowWinOnTop(TWindow *win);
void              __FASTCALL__ twSnapShot(TWindow *win);
TWindow *         __FASTCALL__ twUseWin(TWindow *);
TWindow *         __FASTCALL__ twUsedWin( void );
void              __FASTCALL__ twMoveWin(TWindow *,tAbsCoord dx, tAbsCoord dy);
void              __FASTCALL__ twResizeWin(TWindow *,tAbsCoord width, tAbsCoord height);
void              __FASTCALL__ twCentredWin(TWindow *it,const TWindow *parent); /* parent may be NULL */

void              __FASTCALL__ twFreezeWin(TWindow *);
void              __FASTCALL__ twRefreshLine(TWindow *,tRelCoord y);
void              __FASTCALL__ twRefreshPiece(TWindow *,tRelCoord stx, tRelCoord endx,tRelCoord y);
void              __FASTCALL__ twRefreshWin(TWindow *);
void              __FASTCALL__ twRefreshFullWin(TWindow *);

void *            __FASTCALL__ twGetUsrData(TWindow *);
void *            __FASTCALL__ twSetUsrData(TWindow *,void *);

void              __FASTCALL__ twGetWinPos(TWindow *win,tAbsCoord *x1,tAbsCoord *y1,tAbsCoord *x2,tAbsCoord *y2);
TWindow *         __FASTCALL__ twGetWinAtPos(tAbsCoord x, tAbsCoord y);

#define TW_DEF_FILLER 0x20     /* default filler char */

void              __FASTCALL__ twClearWin(void);
void              __FASTCALL__ twClearWinEx(unsigned char filler);
void              __FASTCALL__ twClrEOL(void);
void              __FASTCALL__ twClrEOLEx(unsigned char filler);

ColorAttr         __FASTCALL__ twSetColor(Color fore,Color back);
ColorAttr         __FASTCALL__ twSetColorAttr(ColorAttr);
void              __FASTCALL__ twGetColor(Color *fore, Color *back );
ColorAttr         __FASTCALL__ twGetColorAttr( void );
Color             __FASTCALL__ twTextColor( Color col );
Color             __FASTCALL__ twTextBkGnd( Color col );
void              __FASTCALL__ twGetFrame(TWindow *win,char *frame,Color* fore,Color* back);
tTitleMode        __FASTCALL__ twGetTitle(TWindow *win,char* title,unsigned cb_title,Color* fore,Color* back);
tTitleMode        __FASTCALL__ twGetFooter(TWindow *win,char* footer,unsigned cb_footer,Color* fore,Color* back);
void              __FASTCALL__ twGetFrameAttr(TWindow *win,char *frame,ColorAttr* attr);
tTitleMode        __FASTCALL__ twGetTitleAttr(TWindow *win,char* title,unsigned cb_title,ColorAttr* attr);
tTitleMode        __FASTCALL__ twGetFooterAttr(TWindow *win,char* footer,unsigned cb_footer,ColorAttr* attr);
void              __FASTCALL__ twSetFrame(TWindow *win,const char *frame,Color fore,Color back);
void              __FASTCALL__ twSetTitle(TWindow *win,const char* title,tTitleMode titlemode,Color fore,Color back);
void              __FASTCALL__ twSetFooter(TWindow *win,const char* footer,tTitleMode footermode,Color fore,Color back);
void              __FASTCALL__ twSetFrameAttr(TWindow *win,const char *frame,ColorAttr attr);
void              __FASTCALL__ twSetTitleAttr(TWindow *win,const char* title,tTitleMode titlemode,ColorAttr attr);
void              __FASTCALL__ twSetFooterAttr(TWindow *win,const char* footer,tTitleMode footermode,ColorAttr attr);

#define TW_CUR_OFF   0
#define TW_CUR_NORM  1
#define TW_CUR_SOLID 2

void              __FASTCALL__ twSetCursorType(int type);
int               __FASTCALL__ twGetCursorType( void );

void              __FASTCALL__ twGotoXY(tRelCoord x,tRelCoord y);
tRelCoord         __FASTCALL__ twWhereX( void );
tRelCoord         __FASTCALL__ twWhereY( void );
void              __FASTCALL__ twPutChar(char ch);
char              __FASTCALL__ twGetChar( void );
int               __FASTCALL__ twPutS(const char *);
int               __FASTCALL__ twDirectWrite(tRelCoord x,tRelCoord y,const void *,unsigned len);
int               __FASTCALL__ twPrintF(const char *fmt,...);
void              __FASTCALL__ twWriteBuffer(TWindow *win,tRelCoord x,tRelCoord y,const void *,unsigned len);
void              __FASTCALL__ twReadBuffer(TWindow *win,tRelCoord x,tRelCoord y,void *,unsigned len);

void              __FASTCALL__ twRemapColor(Color color,unsigned char value);
unsigned char     __FASTCALL__ twGetMappedColor(Color color);

#ifdef __cplusplus
}
#endif

#endif






