/****************************************************************************\
*   DISASM - this header file is part of Binary vIEW project (BIEW)          *
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
*   Copyrights:           1995, 2000 Nick Kurshev                            *
*   License:              See below                                          *
*   Author and developer: Nick Kurshev                                       *
*   Requirement:          ANSI C compatible development system               *
*   Original file name:   plugins/mod/disasm.h                               *
*---------------------- [ D e s c r i p t i o n ] ---------------------------*
*  This file contain function prototypes for dissasembler.                   *
*-------------------------- [ L i c e n c e ] -------------------------------*
* The Binary vIEW (BIEW) is copyright (C) 1995 Nick Kurshev.                 *
* All rights reserved. This software is redistributable under the licence    *
* given in the file "Licence.en" ("Licence.ru" in russian translation)       *
* distributed in the BIEW archive.                                           *
\****************************************************************************/

#ifndef __DISASM_H
#define __DISASM_H

#ifndef __TWIN_H
#include "lib/twin.h"
#endif

#ifndef __COLORSET__H
#include "colorset.h"
#endif

#ifndef __FILE_INI_RUNTIME_SUPPORT_SYSTEM__
#include "lib/file_ini.h"
#endif

/* List of CPU platform. */

#define DISASM_CPU_IX86          0
#define DISASM_DATA              1

#define DISASM_DEFAULT          -1


typedef tBool __FASTCALL__ (*DisasmAction)( void );
typedef unsigned char * MBuffer;

/*
   This struct is ordered as it documented in Athlon manual
   Publication # 22007 Rev: D
*/
typedef struct tagDisasmRet
{
  unsigned long pro_clone;
  char         *str;
  unsigned char codelen;
}DisasmRet;

#define ASM_NOERR  0
#define ASM_SYNTAX 1

/*
   This struct is ordered as it documented in Athlon manual
   Publication # 22007 Rev: D
*/
typedef struct tagAsmRet
{
  MBuffer       insn;
  int           err_code;
  unsigned char insn_len;
}AsmRet;

/* Flags of disassembler */
#define __DISF_NORMAL   0x0000
#define __DISF_SIZEONLY 0x8000

typedef DisasmRet __FASTCALL__ (*DisasmFunc)(unsigned long shift,
                                             MBuffer insn_buff,
                                             unsigned flags);
typedef AsmRet    __FASTCALL__ (*AsmFunc)(const char *str);

typedef struct tag_REGISTRY_DISASM
{
  const char * name;         /* disassembler name */
  const char * prompt[5];    /* prompt on Ctrl-(F1,F3-F5,F10) */
  DisasmAction action[5];    /* actions on Ctrl-(F1,F3-F5,F10) */
  DisasmFunc   disasm;       /* main function of disasm */
  AsmFunc      asm_f;        /* assembler (vice versa of disasm) */
  void         __FASTCALL__ (*ShowShortHelp)(void);
  int          __FASTCALL__ (*max_insn_len)(void); /* Max length of 1 disasm instruction */
  ColorAttr    __FASTCALL__ (*GetInsnColor)(unsigned long clone);
  int          __FASTCALL__ (*GetDefBitness)(void);
  char         __FASTCALL__ (*CloneShortName)(unsigned long clone);
  void         __FASTCALL__ (*init)(void);
  void         __FASTCALL__ (*term)(void);
  void         __FASTCALL__ (*read_ini)(hIniProfile *);
  void         __FASTCALL__ (*save_ini)(hIniProfile *);
}REGISTRY_DISASM;

extern REGISTRY_DISASM *activeDisasm;
#define PANMOD_FULL 2
#define PANMOD_MEDIUM 1
#define PANMOD_WIDE 0
extern unsigned disPanelMode;

extern DisasmRet Disassembler(unsigned long ulShift,MBuffer buffer,unsigned flags);

/* Common disassembler utility */

extern char * __FASTCALL__ TabSpace(char * str,unsigned nSpace);
extern void   __FASTCALL__ disSetModifier(char *str,const char *modf);

#define DISARG_LONG  -4
#define DISARG_SHORT -2
#define DISARG_CHAR  -1
#define DISARG_BYTE  1
#define DISARG_WORD  2
#define DISARG_DWORD 4
/*
   str     - return string
   flags   - as described in reg_form.h
   ulShift - offset to field for binding
   codelen - length of field for binding
   defval  - default value if not binding
   type    - see above
   Examples:
   *** 1 ***
   00005678: B8 34 12       mov   ax, 1234
   after references resolving it can will:
                            mov   ax, off16 KERNEL.@4
   i.e. i must call this function as:
   strcpy(outstr,"mov ax,");
   disAppendDigits(outstr, 0x5679, 1, 2, 0x1234, DISARG_WORD);

   *** 2 ***
   00005678: 8B 80 34 12       mov   ax, [bx+si+1234]
   after references resolving it can will:
                               mov   ax, [bx+si+off16 KERNEL.@4]
   i.e. i must call this function as:
   strcpy(outstr,"mov ax,[bx+si+");
   disAppendDigits(outstr, 0x5680, 1, 2, 0x1234, DISARG_WORD);
   strcat(outstr,"]");
*/
extern void __FASTCALL__  disAppendDigits(char *str,unsigned long ulShift,int flags,
                              char codelen,void *defval,int type);

#define DISADR_SHORT   0x00
#define DISADR_NEAR16  0x01
#define DISADR_NEAR32  0x02
#define DISADR_USESEG  0x04
#define DISADR_FAR16   (DISADR_USESEG | DISADR_NEAR16)
#define DISADR_FAR32   (DISADR_USESEG | DISADR_NEAR32)
/*
   str     - return string
   ulShift - offset to field for binding
   codelen - length of field for binding
   distin  - original value of field (like defval)
   r_sh    - real shift to begin of insn (for pass to CodeGuider as return addr)
   type    - see above
   seg     - segment value (optional)
   Examples:
   *** 1 ***
   00001002: 9A 78 56 34 12       callf 1234:5678
   after references resolving it may:
                                  callf KERNEL.@90
   i.e. i must call this function as:
   strcpy(outstr,"callf ");
   disAppendFAddr(outstr, 0x1003, 0x5678, 0x1002, DISADR_FAR16, 0x1234, 4);
   *** 2 ***
   00001002: 66 E9 78 56 34 12       calln32 12345678
   after references resolving it may:
                                     calln32 KERNEL32.TerminateApp
   i.e. i must call this function as:
   strcpy(outstr,"calln32 ");
   disAppendFAddr(outstr, 0x1004, 0x12345678, 0x1002, DISADR_NEAR32, 0, 4);
*/
extern void __FASTCALL__  disAppendFAddr(char * str,long ulShift,long distin,
                             unsigned long r_sh,char type,
                             unsigned seg,char codelen);

#endif
